﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Data.Entity
Imports System.Windows.Data
Imports System.Windows.Input
Imports Model

Namespace Workspaces
    Public Class UrgencyListVm
        Inherits ViewModelBase

        Private _context As OrdersContext.OrdersDbContext
        Private _aviableWorkCenters As List(Of WorkCenter)
        Private ReadOnly _statusBarVm As New StatusVm

        Public Sub New()
            'Der Konstructor ist im Grunde nur für den Designer
            If IsInDesignMode Then
                LoadDataForDesignMode()
            Else
                Throw New Exception("Der Konstructor ist nur für den Designer. Bitte verwende einen der anderen.")
            End If
        End Sub

        Public Sub New(statusBar As StatusVm)
            If Not IsInDesignMode Then
                _statusBarVm = statusBar
                LoadData()
            Else
                _statusBarVm = statusBar
                LoadDataForDesignMode()
            End If
        End Sub

        Private Sub LoadDataForDesignMode()
            Dim answere As New Answer() With {.AnswerText = "Antworttext", .ContactName = "Neue Kontakt", .RequestDate = DateTime.Now}
            Dim answereList As New List(Of Answer) : answereList.Add(answere)
            UrgentOrderList = New ObservableCollection(Of UrgentOrderVm)
            _aviableWorkCenters = New List(Of WorkCenter)
            _aviableWorkCenters.Add(New WorkCenter("DesignWorkCenter", 5))

            UrgentOrderList.Add(New UrgentOrderVm(New Order() With {.ConfirmationDate = DateTime.Now, .ArticleNr = 12345, .Description = "Testartikel", .Remarks = "Beschreibung" _
                                       , .Answers = answereList, .OrderNr = 521145, .Quantity = 235, .CurrentWorkCenter = _aviableWorkCenters.First}, _aviableWorkCenters))
            UrgentOrdersView = CollectionViewSource.GetDefaultView(UrgentOrderList)

        End Sub

        Private Function FilterOrders(obj As Object) As Boolean
            If ShowDeleted Then
                Return True
            Else
                Return Not DirectCast(obj, UrgentOrderVm).IsDeleted
            End If
        End Function

        Private Sub LoadData()
            _context = New OrdersContext.OrdersDbContext
            _aviableWorkCenters = _context.WorkCenters.Where(Function(wc) wc.DeletedFlag = False).ToList

            Dim ordersList = (From u As Order In _context.Orders.Include(Function(u) u.Answers).Include(Function(wc) wc.CurrentWorkCenter)).ToList
            UrgentOrderList = New ObservableCollection(Of UrgentOrderVm)
            For Each order In ordersList
                UrgentOrderList.Add(New UrgentOrderVm(order, _aviableWorkCenters))
            Next

            'Include sorgt dafür das benötigte Daten (Answers) gleich mitgeladen werden anstatt X Befehle an die DB zu senden was viel länger dauert
            'Um in Include keinen String sondern einen LINQ mitgeben zu können ist ein Import auf System.Data.Entity notwendig
            UrgentOrdersView = CollectionViewSource.GetDefaultView(UrgentOrderList)
            UrgentOrdersView.Filter = AddressOf FilterOrders

            UrgentOrdersView.MoveCurrentToFirst() 'Ersten Datensatz mal markieren
            _statusBarVm.LastReloadDateTime = DateTime.Now
        End Sub


#Region "Properties"

        Private _urgentOrderList As ObservableCollection(Of UrgentOrderVm)
        Friend Property UrgentOrderList() As ObservableCollection(Of UrgentOrderVm)
            Get
                Return _urgentOrderList
            End Get
            Set(ByVal value As ObservableCollection(Of UrgentOrderVm))
                _urgentOrderList = value
                RaisePropertyChanged()
            End Set
        End Property


        Private _urgentOrdersView As ICollectionView
        Public Property UrgentOrdersView() As ICollectionView
            Get
                Return _urgentOrdersView
            End Get
            Set(ByVal value As ICollectionView)
                _urgentOrdersView = value
                RaisePropertyChanged()
            End Set
        End Property


        Private _showDeleted As Boolean
        Public Property ShowDeleted() As Boolean
            Get
                Return _showDeleted
            End Get
            Set(ByVal value As Boolean)
                _showDeleted = value
                RaisePropertyChanged()
                UrgentOrdersView.Refresh()
            End Set
        End Property

#End Region

#Region "Commands"

        Private _addOrderCommand As ICommand
        Public Property AddOrderCommand() As ICommand
            Get
                If _addOrderCommand Is Nothing Then _addOrderCommand = New RelayCommand(AddressOf AddOrderCommand_Execute)
                Return _addOrderCommand
            End Get
            Set(ByVal value As ICommand)
                _addOrderCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Sub AddOrderCommand_Execute(obj As Object)
            Dim diaServ = Services.ServiceContainer.GetService(Of Services.IDialogWindowService)
            Dim newOrderVm As New UrgentOrderVm(New [Order], _aviableWorkCenters)
            If diaServ.ShowModalDialog("newOrder", newOrderVm) Then
                _context.Orders.Add(newOrderVm.GetModelOrderObject)
                UrgentOrderList.Add(newOrderVm)
            End If
        End Sub


        Private _editOrderCommand As ICommand
        Public Property EditOrderCommand() As ICommand
            Get
                If _editOrderCommand Is Nothing Then _editOrderCommand = New RelayCommand(AddressOf EditOrderCommand_Execute, AddressOf EditOrderCommand_CanExecute)
                Return _editOrderCommand
            End Get
            Set(ByVal value As ICommand)
                _editOrderCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Function EditOrderCommand_CanExecute(obj As Object) As Boolean
            Return UrgentOrdersView.CurrentItem IsNot Nothing
        End Function

        Private Sub EditOrderCommand_Execute(obj As Object)
            Dim msgServ = Services.ServiceContainer.GetService(Of Services.IMessageboxService)
            If _context.ChangeTracker.HasChanges Then
                If msgServ.Show("Du solltest zuerst speichern bevor du bearbeitest. Jetzt speichern?", "Arbeit speichern", Services.EnuMessageBoxButton.YesCancel, Services.EnuMessageBoxImage.Question) = Services.EnuMessageBoxResult.Yes Then
                    SaveCommand.Execute(Nothing)
                    OpenEditDialog()
                End If
            Else
                OpenEditDialog()
            End If
        End Sub


        Private Sub OpenEditDialog()
            Dim diaServ = Services.ServiceContainer.GetService(Of Services.IDialogWindowService)
            Dim editOrderVm = DirectCast(UrgentOrdersView.CurrentItem, UrgentOrderVm)
            If Not diaServ.ShowModalDialog("editOrder", editOrderVm) Then
                Debug.WriteLine("Dialog abgebrochen")
                LoadData()
            End If
        End Sub

        Private _deleteOrderCommand As ICommand
        Public Property DeleteOrderCommand() As ICommand
            Get
                If _deleteOrderCommand Is Nothing Then _deleteOrderCommand = New RelayCommand(AddressOf DeleteOrderCommand_Execute,AddressOf DeleteOrderCommand_CanExecute)
                Return _deleteOrderCommand
            End Get
            Set(ByVal value As ICommand)
                _deleteOrderCommand = value
            End Set
        End Property

        Private Sub DeleteOrderCommand_Execute(obj As Object)
            DirectCast(UrgentOrdersView.CurrentItem, UrgentOrderVm).IsDeleted = True
            UrgentOrdersView.Refresh()
        End Sub

        Private Function DeleteOrderCommand_CanExecute(obj As Object) As Boolean
            Return UrgentOrdersView.CurrentItem IsNot Nothing
        End Function

        Private _saveCommand As ICommand
        Public Property SaveCommand() As ICommand
            Get
                If _saveCommand Is Nothing Then _saveCommand = New RelayCommand(AddressOf SaveCommand_Execute, AddressOf SaveCommand_CanExecute)
                Return _saveCommand
            End Get
            Set(ByVal value As ICommand)
                _saveCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private _refreshCommand As ICommand
        Public Property RefreshCommand() As ICommand
            Get
                If _refreshCommand Is Nothing Then _refreshCommand = New RelayCommand(AddressOf RefreshCommand_Execute)
                Return _refreshCommand
            End Get
            Set(ByVal value As ICommand)
                _refreshCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Sub RefreshCommand_Execute(obj As Object)
            Dim msgServ = Services.ServiceContainer.GetService(Of Services.IMessageboxService)
            If _context.ChangeTracker.HasChanges Then
                If msgServ.Show("Er gibt ungespeicherter Objekte, wirklich neu laden und änderungen verwerfen?", "Änderungen verwerfen?", Services.EnuMessageBoxButton.YesCancel, Services.EnuMessageBoxImage.Question) = Services.EnuMessageBoxResult.Yes Then
                    LoadData()
                End If
            Else
                LoadData()
            End If

        End Sub

        Private Sub SaveCommand_Execute(obj As Object)
            Dim msgServ = Services.ServiceContainer.GetService(Of Services.IMessageboxService)
            msgServ.Show(_context.SaveChanges & " Änderungen gespeichert", "Speicherstatus", Services.EnuMessageBoxButton.Ok, Services.EnuMessageBoxImage.Information)
        End Sub

        Private Function SaveCommand_CanExecute(obj As Object) As Boolean
            If _context Is Nothing Then Return False
            _context.ChangeTracker.DetectChanges()
            Return _context.ChangeTracker.HasChanges()
        End Function

#End Region

    End Class
End Namespace