﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Windows.Data
Imports System.Windows.Input
Imports Model

Namespace Workspaces
    Public Class UrgentOrderVm
        Inherits ViewModelBase

        Private ReadOnly _urgentOrder As Order
        Private Const Desc As String = "Fülle die Felder aus und die Bestellung zu bearbeiten."

        Public Sub New()
            _urgentOrder = New Order
            Answers = New ObservableCollection(Of AnswerVm)
            AnswersView = CollectionViewSource.GetDefaultView(Answers)
            CurrentWorkCenterVm = New WorkCenterVm
            AviableWorkCenters = New ObservableCollection(Of WorkCenterVm)

            VmTitle = "Neue Bestellung"
            VmDescription = Desc
        End Sub

        Public Sub New(uOrder As Order, workCenters As List(Of WorkCenter))
            _urgentOrder = uOrder
            If _urgentOrder.Answers Is Nothing Then _urgentOrder.Answers = New List(Of Answer)

            AviableWorkCenters = New ObservableCollection(Of WorkCenterVm)
            For Each wc In workCenters
                AviableWorkCenters.Add(New WorkCenterVm(wc))
            Next

            Answers = New ObservableCollection(Of AnswerVm)

            For Each ans In _urgentOrder.Answers
                Answers.Add(New AnswerVm(ans))
            Next
            AnswersView = CollectionViewSource.GetDefaultView(Answers)
            AnswersView.Filter = AddressOf AnswerFilter
            If _urgentOrder.CurrentWorkCenter IsNot Nothing Then CurrentWorkCenterVm = AviableWorkCenters.Where(Function(ws) ws.Id = _urgentOrder.CurrentWorkCenter.Id).First

            VmTitle = "Bestellung " & _urgentOrder.Id.ToString & " bearbeiten"
            VmDescription = Desc
        End Sub

        Private Function AnswerFilter(obj As Object) As Boolean
            If IncludeDeletedAnswers Then
                Return True
            Else
                Return Not DirectCast(obj, AnswerVm).IsDeleted
            End If
        End Function



#Region "Propertys"

        Private _awaibleWorkCenters As ObservableCollection(Of WorkCenterVm)
        Public Property AviableWorkCenters() As ObservableCollection(Of WorkCenterVm)
            Get
                Return _awaibleWorkCenters
            End Get
            Set
                _awaibleWorkCenters = Value
                RaisePropertyChanged()
            End Set
        End Property

        Public Property UrgentOrderId As Integer
            Get
                Return _urgentOrder.Id
            End Get
            Set
                _urgentOrder.Id = Value
                RaisePropertyChanged()
            End Set
        End Property

        Public Property OrderNbr As Integer?
            Get
                Return _urgentOrder.OrderNr
            End Get
            Set
                _urgentOrder.OrderNr = Value
                RaisePropertyChanged()
            End Set
        End Property

        Public Property OrderArticle As Integer?
            Get
                Return _urgentOrder.ArticleNr
            End Get
            Set
                _urgentOrder.ArticleNr = Value
                RaisePropertyChanged()
            End Set
        End Property

        Public Property OrderQuantity As Integer
            Get
                Return _urgentOrder.Quantity
            End Get
            Set
                _urgentOrder.Quantity = Value
                RaisePropertyChanged()
            End Set
        End Property

        Public Property OrderDescription As String
            Get
                Return _urgentOrder.Description
            End Get
            Set
                _urgentOrder.Description = Value
                RaisePropertyChanged()
            End Set
        End Property

        Public Property Remarks As String
            Get
                Return _urgentOrder.Remarks
            End Get
            Set
                _urgentOrder.Remarks = Value
                RaisePropertyChanged()
            End Set
        End Property

        Public Property ConfirmationDate As Date?
            Get
                Return _urgentOrder.ConfirmationDate
            End Get
            Set
                _urgentOrder.ConfirmationDate = Value
                RaisePropertyChanged()
            End Set
        End Property




        Public Property IsDeleted() As Boolean
            Get
                Return _urgentOrder.DeletedFlag
            End Get
            Set(ByVal value As Boolean)
                _urgentOrder.DeletedFlag = value
                If value Then
                    _urgentOrder.DeletedTimestamp = DateTime.Now
                Else
                    _urgentOrder.DeletedTimestamp = Nothing
                End If
                RaisePropertyChanged()
                RaisePropertyChanged("DeletedTimestamp")
            End Set
        End Property

        Public ReadOnly Property DeletedTimestamp() As DateTime?
            Get
                Return _urgentOrder.DeletedTimestamp
            End Get
        End Property


        Public ReadOnly Property LastUpdateTimestamp As DateTime?
            Get
                Return _urgentOrder.LastUpdateTimestamp
            End Get
        End Property

        Public ReadOnly Property CreationTimestamp As DateTime?
            Get
                Return _urgentOrder.CreationTimestamp
            End Get
        End Property

        Public ReadOnly Property CreatedBy As String
            Get
                Return _urgentOrder.CreatedBy
            End Get
        End Property

        Public ReadOnly Property CreatedOn As String
            Get
                Return _urgentOrder.CreatedOn
            End Get
        End Property


        Public ReadOnly Property LastUpdateBy As String
            Get
                Return _urgentOrder.LastUpdateBy
            End Get
        End Property

        Public ReadOnly Property LastUpdateOn As String
            Get
                Return _urgentOrder.LastUpdateOn
            End Get
        End Property


        Private _currentWorkCenterVm As WorkCenterVm
        Public Property CurrentWorkCenterVm() As WorkCenterVm
            Get
                Return _currentWorkCenterVm
            End Get
            Set(ByVal value As WorkCenterVm)
                _currentWorkCenterVm = value
                If _urgentOrder IsNot Nothing Then
                    _urgentOrder.CurrentWorkCenter = _currentWorkCenterVm.GetWorkCenterModelObject
                End If
                RaisePropertyChanged()
            End Set
        End Property


        Private _answers As ObservableCollection(Of AnswerVm)
        Friend Property Answers() As ObservableCollection(Of AnswerVm)
            Get
                Return _answers
            End Get
            Set(ByVal value As ObservableCollection(Of AnswerVm))
                _answers = value
                RaisePropertyChanged()
                RaisePropertyChanged("CanAddAnswers")
            End Set
        End Property

        Private _answersView As ICollectionView
        Public Property AnswersView() As ICollectionView
            Get
                Return _answersView
            End Get
            Set(ByVal value As ICollectionView)
                _answersView = value
                RaisePropertyChanged()
            End Set
        End Property

        Private _includeDeletedAnswers As Boolean
        Public Property IncludeDeletedAnswers() As Boolean
            Get
                Return _includeDeletedAnswers
            End Get
            Set(ByVal value As Boolean)
                _includeDeletedAnswers = value
                RaisePropertyChanged()
                AnswersView.Refresh()
            End Set
        End Property


        Private _addAnswerCommand As ICommand
        Public Property AddAnswerCommand() As ICommand
            Get
                If _addAnswerCommand Is Nothing Then _addAnswerCommand = New RelayCommand(AddressOf AddOrderCommand_Execute)
                Return _addAnswerCommand
            End Get
            Set(ByVal value As ICommand)
                _addAnswerCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Sub AddOrderCommand_Execute(obj As Object)
            Dim diaServ = Services.ServiceContainer.GetService(Of Services.IDialogWindowService)
            Dim newAnswerVm As New AnswerVm(New Answer)
            If diaServ.ShowModalDialog("newAnswer", newAnswerVm) Then
                If _urgentOrder.Answers Is Nothing Then _urgentOrder.Answers = New List(Of Answer)
                _urgentOrder.Answers.Add(newAnswerVm.GetModelAnswerObject)
                _answers.Add(newAnswerVm)
            End If
        End Sub


#End Region

        Friend Function GetModelOrderObject() As Order
            Return _urgentOrder
        End Function



    End Class
End Namespace